# frozen_string_literal: false
#
#  trofs support
#                               by Hidetoshi NAGAI (nagai@ai.kyutech.ac.jp)
#

# call setup script for general 'tkextlib' libraries
require 'tkextlib/setup.rb'

# call setup script
require 'tkextlib/trofs/setup.rb'

# load library
require 'tkextlib/trofs/trofs.rb'
